#!/usr/bin/env python

"""
An iterator merging class similar to heapq.merge in Python 2.6.

Copyright (C) 2009 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from bisect import insort_right

class itermerge:

    """
    Merge ordered sequences to produce an ordered, combined sequence of
    results.
    """

    def __init__(self, sequences):
        self.iters = []

        # Prepare the underlying iterators.

        for seq in sequences:
            it = iter(seq)
            next = it.next
            self._add_next(next)

    def sort(self):
        pass # The output should be sorted.

    def __iter__(self):
        return self

    def _add_next(self, next):

        """
        Store the current value for an iterator, alongside the means of
        getting the next value: the 'next' method.
        """

        try:
            insort_right(self.iters, (next(), next))
        except StopIteration:
            pass

    def next(self):
        if self.iters:
            value, next = self.iters[0]
            del self.iters[0]
            self._add_next(next)
            return value
        else:
            raise StopIteration

# vim: tabstop=4 expandtab shiftwidth=4
